"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const testUtils_1 = require("./utils/testUtils");
const Note_1 = require("@joplin/lib/models/Note");
const Folder_1 = require("@joplin/lib/models/Folder");
const app_1 = require("./app");
const trash_1 = require("@joplin/lib/services/trash");
const Command = require('./command-rmnote');
const setUpCommand = () => {
    const command = (0, testUtils_1.setupCommandForTesting)(Command);
    const promptMock = jest.fn(() => true);
    command.setPrompt(promptMock);
    return { command, promptMock };
};
const createTestNote = () => __awaiter(void 0, void 0, void 0, function* () {
    const folder = yield Folder_1.default.save({ title: 'folder' });
    (0, app_1.default)().switchCurrentFolder(folder);
    return yield Note_1.default.save({ title: 'note1', parent_id: folder.id });
});
describe('command-rmnote', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
        yield (0, testUtils_1.setupApplication)();
    }));
    it('should move to the trash by default, without prompting', () => __awaiter(void 0, void 0, void 0, function* () {
        const { id: noteId } = yield createTestNote();
        const { command, promptMock } = setUpCommand();
        yield command.action({ 'note-pattern': 'note1', options: {} });
        expect(promptMock).not.toHaveBeenCalled();
        expect((yield Note_1.default.allItemsInTrash()).noteIds.includes(noteId)).toBe(true);
    }));
    it('should permanently delete trashed items by default, with prompting', () => __awaiter(void 0, void 0, void 0, function* () {
        const { id: noteId } = yield createTestNote();
        const { command, promptMock } = setUpCommand();
        // Should not prompt when deleting from a folder
        yield command.action({ 'note-pattern': 'note1', options: {} });
        expect(promptMock).toHaveBeenCalledTimes(0);
        // Should prompt when deleting from trash
        (0, app_1.default)().switchCurrentFolder(yield Folder_1.default.load((0, trash_1.getTrashFolderId)()));
        yield command.action({ 'note-pattern': 'note1', options: {} });
        expect(promptMock).toHaveBeenCalledTimes(1);
        expect(yield Note_1.default.load(noteId, { includeDeleted: true })).toBe(undefined);
    }));
});
//# sourceMappingURL=command-rmnote.test.js.map