"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const test_utils_1 = require("@joplin/lib/testing/test-utils");
const testUtils_1 = require("./utils/testUtils");
const Folder_1 = require("@joplin/lib/models/Folder");
const Note_1 = require("@joplin/lib/models/Note");
const Command = require('./command-rmbook');
const setUpCommand = () => {
    const command = (0, testUtils_1.setupCommandForTesting)(Command);
    const promptMock = jest.fn(() => true);
    command.setPrompt(promptMock);
    return { command, promptMock };
};
describe('command-rmbook', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
        yield (0, testUtils_1.setupApplication)();
    }));
    it('should ask before moving to the trash', () => __awaiter(void 0, void 0, void 0, function* () {
        yield Folder_1.default.save({ title: 'folder1' });
        const { command, promptMock } = setUpCommand();
        yield command.action({ 'notebook': 'folder1', options: {} });
        expect(promptMock).toHaveBeenCalledTimes(1);
        const folder1 = yield Folder_1.default.loadByTitle('folder1');
        expect(folder1.deleted_time).not.toBeFalsy();
        expect((yield Note_1.default.allItemsInTrash()).folderIds).toHaveLength(1);
    }));
    it('cancelling a prompt should prevent deletion', () => __awaiter(void 0, void 0, void 0, function* () {
        yield Folder_1.default.save({ title: 'folder1' });
        const { command, promptMock } = setUpCommand();
        promptMock.mockImplementation(() => false);
        yield command.action({ 'notebook': 'folder1', options: {} });
        expect((yield Note_1.default.allItemsInTrash()).folderIds).toHaveLength(0);
    }));
    it('should not prompt when the force flag is given', () => __awaiter(void 0, void 0, void 0, function* () {
        const { id: folder1Id } = yield Folder_1.default.save({ title: 'folder1' });
        const { id: folder2Id } = yield Folder_1.default.save({ title: 'folder2', parent_id: folder1Id });
        const { command, promptMock } = setUpCommand();
        yield command.action({ 'notebook': 'folder1', options: { force: true } });
        expect(promptMock).toHaveBeenCalledTimes(0);
        expect((yield Note_1.default.allItemsInTrash()).folderIds.includes(folder1Id)).toBe(true);
        expect((yield Note_1.default.allItemsInTrash()).folderIds.includes(folder2Id)).toBe(true);
    }));
    it('should support permanent deletion', () => __awaiter(void 0, void 0, void 0, function* () {
        const { id: folder1Id } = yield Folder_1.default.save({ title: 'folder1' });
        const { id: folder2Id } = yield Folder_1.default.save({ title: 'folder2' });
        const { command, promptMock } = setUpCommand();
        yield command.action({ 'notebook': 'folder1', options: { permanent: true, force: true } });
        expect(promptMock).not.toHaveBeenCalled();
        // Should be permanently deleted.
        expect((yield Note_1.default.allItemsInTrash()).folderIds.includes(folder1Id)).toBe(false);
        expect(yield Folder_1.default.load(folder1Id, { includeDeleted: true })).toBe(undefined);
        // folder2 should not be deleted
        expect(yield Folder_1.default.load(folder2Id, { includeDeleted: false })).toBeTruthy();
        // Should prompt before deleting
        yield command.action({ 'notebook': 'folder2', options: { permanent: true } });
        expect(promptMock).toHaveBeenCalled();
        expect(yield Folder_1.default.load(folder2Id, { includeDeleted: false })).toBeUndefined();
    }));
});
//# sourceMappingURL=command-rmbook.test.js.map