"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const AsyncActionQueue_1 = require("./AsyncActionQueue");
describe('AsyncActionQueue', () => {
    beforeEach(() => {
        jest.useRealTimers();
    });
    test('should run a single task', () => __awaiter(void 0, void 0, void 0, function* () {
        const queue = new AsyncActionQueue_1.default(0);
        yield expect(new Promise(resolve => {
            queue.push(() => resolve('success'));
        })).resolves.toBe('success');
        yield expect(new Promise(resolve => {
            queue.push(() => resolve('task 2'));
        })).resolves.toBe('task 2');
    }));
    test('should merge all tasks by default', () => __awaiter(void 0, void 0, void 0, function* () {
        const queue = new AsyncActionQueue_1.default(100);
        jest.useFakeTimers();
        const result = {
            ranFirst: false,
            ranSecond: false,
            ranThird: false,
            ranFourth: false,
        };
        queue.push(() => {
            result.ranFirst = true;
        });
        queue.push(() => {
            result.ranSecond = true;
        });
        queue.push(() => {
            result.ranThird = true;
        });
        queue.push(() => {
            result.ranFourth = true;
        });
        const processPromise = queue.processAllNow();
        yield jest.runAllTimersAsync();
        yield processPromise;
        expect(result).toMatchObject({
            ranFirst: false,
            ranSecond: false,
            ranThird: false,
            ranFourth: true,
        });
    }));
    test.each([
        {
            tasks: [
                'a', 'b',
            ],
            expectedToRun: [
                'a', 'b',
            ],
        },
        {
            tasks: [
                'a', 'b', 'c',
            ],
            expectedToRun: [
                'a', 'b', 'c',
            ],
        },
        {
            tasks: [
                'group1', 'group1', 'group1', 'group2', 'group1', 'group1', 'group2', 'group2',
            ],
            expectedToRun: [
                'group1', 'group2', 'group1', 'group2',
            ],
        },
    ])('should support customising how tasks are merged', ({ tasks, expectedToRun }) => __awaiter(void 0, void 0, void 0, function* () {
        const queue = new AsyncActionQueue_1.default(100);
        // Determine which tasks can be merged based on their context
        queue.setCanSkipTaskHandler((current, next) => {
            return current.context === next.context;
        });
        jest.useFakeTimers();
        const result = [];
        for (const task of tasks) {
            queue.push(() => {
                result.push(task);
            }, task);
        }
        const processPromise = queue.processAllNow();
        yield jest.runAllTimersAsync();
        yield processPromise;
        expect(result).toMatchObject(expectedToRun);
    }));
});
//# sourceMappingURL=AsyncActionQueue.test.js.map