"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderTags = exports.isTagSelected = exports.renderFolders = exports.isFolderSelected = void 0;
const Folder_1 = require("../../models/Folder");
const BaseModel_1 = require("../../BaseModel");
const trash_1 = require("../../services/trash");
const getCollator_1 = require("../../models/utils/getCollator");
function folderHasChildren_(folders, folderId) {
    if (folderId === (0, trash_1.getTrashFolderId)()) {
        return !!folders.find(f => !!f.deleted_time);
    }
    for (let i = 0; i < folders.length; i++) {
        const folder = folders[i];
        const folderParentId = (0, trash_1.getDisplayParentId)(folder, folders.find(f => f.id === folder.parent_id));
        if (folderParentId === folderId)
            return true;
    }
    return false;
}
function folderIsCollapsed(folders, folderId, collapsedFolderIds) {
    if (!collapsedFolderIds || !collapsedFolderIds.length)
        return false;
    while (true) {
        const folder = BaseModel_1.default.byId(folders, folderId);
        if (!folder)
            throw new Error(`No folder with id ${folder.id}`);
        const folderParentId = (0, trash_1.getDisplayParentId)(folder, folders.find(f => f.id === folder.parent_id));
        if (!folderParentId)
            return false;
        if (collapsedFolderIds.indexOf(folderParentId) >= 0)
            return true;
        folderId = folderParentId;
    }
}
const isFolderSelected = (folder, context) => {
    return context.selectedFolderId === folder.id && context.notesParentType === 'Folder';
};
exports.isFolderSelected = isFolderSelected;
function renderFoldersRecursive_(props, renderItem, items, parentId, depth, order) {
    const folders = props.folders;
    for (let i = 0; i < folders.length; i++) {
        const folder = folders[i];
        const folderParentId = (0, trash_1.getDisplayParentId)(folder, props.folders.find(f => f.id === folder.parent_id));
        if (!Folder_1.default.idsEqual(folderParentId, parentId))
            continue;
        if (folderIsCollapsed(props.folders, folder.id, props.collapsedFolderIds))
            continue;
        const hasChildren = folderHasChildren_(folders, folder.id);
        order.push(folder.id);
        items.push(renderItem(folder, hasChildren, depth));
        if (hasChildren) {
            const result = renderFoldersRecursive_(props, renderItem, items, folder.id, depth + 1, order);
            items = result.items;
            order = result.order;
        }
    }
    return {
        items: items,
        order: order,
    };
}
const renderFolders = (props, renderItem) => {
    return renderFoldersRecursive_(props, renderItem, [], '', 0, []);
};
exports.renderFolders = renderFolders;
const sortTags = (tags) => {
    tags = tags.slice();
    const collator = (0, getCollator_1.getCollator)();
    tags.sort((a, b) => {
        // It seems title can sometimes be undefined (perhaps when syncing
        // and before tag has been decrypted?). It would be best to find
        // the root cause but for now that will do.
        //
        // Fixes https://github.com/laurent22/joplin/issues/4051
        if (!a || !a.title || !b || !b.title)
            return 0;
        // Note: while newly created tags are normalized and lowercase
        // imported tags might be any case, so we need to do case-insensitive
        // sort.
        return collator.compare(a.title, b.title);
    });
    return tags;
};
const isTagSelected = (tag, context) => {
    return context.selectedTagId === tag.id && context.notesParentType === 'Tag';
};
exports.isTagSelected = isTagSelected;
const renderTags = (unsortedTags, renderItem) => {
    const tags = sortTags(unsortedTags);
    const tagItems = [];
    const order = [];
    for (let i = 0; i < tags.length; i++) {
        const tag = tags[i];
        order.push(tag.id);
        tagItems.push(renderItem(tag));
    }
    return {
        items: tagItems,
        order: order,
    };
};
exports.renderTags = renderTags;
//# sourceMappingURL=side-menu-shared.js.map