"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cancelTimers = exports.scheduleRefreshFolders = exports.refreshFolders = exports.allForDisplay = void 0;
const Folder_1 = require("./models/Folder");
const Setting_1 = require("./models/Setting");
const shim_1 = require("./shim");
const refreshCalls_ = [];
// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
let scheduleRefreshFoldersIID_ = null;
const allForDisplay = (options = {}) => __awaiter(void 0, void 0, void 0, function* () {
    const orderDir = Setting_1.default.value('folders.sortOrder.reverse') ? 'DESC' : 'ASC';
    const folderOptions = Object.assign({ caseInsensitive: true, order: [
            {
                by: 'title',
                dir: orderDir,
            },
        ] }, options);
    let folders = yield Folder_1.default.all(folderOptions);
    if (Setting_1.default.value('folders.sortOrder.field') === 'last_note_user_updated_time') {
        folders = yield Folder_1.default.orderByLastModified(folders, orderDir);
    }
    if (Setting_1.default.value('showNoteCounts')) {
        yield Folder_1.default.addNoteCounts(folders, Setting_1.default.value('showCompletedTodos'));
    }
    return folders;
});
exports.allForDisplay = allForDisplay;
// `selectedFolderId` should be the currently selected folder. Set it to an empty string if that
// information is not available in the current context.
const refreshFolders = (dispatch, selectedFolderId) => __awaiter(void 0, void 0, void 0, function* () {
    refreshCalls_.push(true);
    try {
        const folders = yield (0, exports.allForDisplay)({
            includeConflictFolder: true,
            includeTrash: true,
        });
        dispatch({
            type: 'FOLDER_UPDATE_ALL',
            items: folders,
        });
        // If the currently selected folder no longer exist, select a default folder
        if (selectedFolderId && !folders.find(f => f.id === selectedFolderId)) {
            const defaultFolder = yield Folder_1.default.defaultFolder();
            if (defaultFolder) {
                dispatch({
                    type: 'FOLDER_SELECT',
                    id: defaultFolder.id,
                });
            }
        }
    }
    finally {
        refreshCalls_.pop();
    }
});
exports.refreshFolders = refreshFolders;
const scheduleRefreshFolders = (dispatch, selectedFolderId) => __awaiter(void 0, void 0, void 0, function* () {
    if (scheduleRefreshFoldersIID_)
        shim_1.default.clearTimeout(scheduleRefreshFoldersIID_);
    scheduleRefreshFoldersIID_ = shim_1.default.setTimeout(() => {
        scheduleRefreshFoldersIID_ = null;
        void (0, exports.refreshFolders)(dispatch, selectedFolderId);
    }, 1000);
});
exports.scheduleRefreshFolders = scheduleRefreshFolders;
const cancelTimers = () => __awaiter(void 0, void 0, void 0, function* () {
    if (scheduleRefreshFoldersIID_) {
        shim_1.default.clearTimeout(scheduleRefreshFoldersIID_);
        scheduleRefreshFoldersIID_ = null;
    }
    return new Promise((resolve) => {
        const iid = shim_1.default.setInterval(() => {
            if (!refreshCalls_.length) {
                shim_1.default.clearInterval(iid);
                resolve(null);
            }
        }, 100);
    });
});
exports.cancelTimers = cancelTimers;
//# sourceMappingURL=folders-screen-utils.js.map