"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const time_1 = require("./time");
const Setting_1 = require("./models/Setting");
const path_utils_1 = require("./path-utils");
const md5 = require('md5');
const buffer_1 = require("buffer");
class FsDriverBase {
    stat(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: stat()');
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    readFile(_path, _encoding = 'utf8') {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: readFile');
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    appendFile(_path, _content, _encoding = 'base64') {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: appendFile');
        });
    }
    copy(_source, _dest) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: copy');
        });
    }
    chmod(_source, _mode) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: chmod');
        });
    }
    // Must also create parent directories
    mkdir(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: mkdir');
        });
    }
    unlink(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: unlink');
        });
    }
    move(_source, _dest) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: move');
        });
    }
    rename(source, dest) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.move(source, dest);
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    readFileChunk(_handle, _length, _encoding = 'base64') {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: readFileChunk');
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    open(_path, _mode) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: open');
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    close(_handle) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: close');
        });
    }
    readDirStats(_path, _options = null) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: readDirStats');
        });
    }
    exists(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: exists');
        });
    }
    remove(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: remove');
        });
    }
    setTimestamp(_path, _timestampDate) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: setTimestamp');
        });
    }
    isDirectory(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const stat = yield this.stat(path);
            return !stat ? false : stat.isDirectory();
        });
    }
    writeFile(_path, _content, _encoding = 'base64') {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented');
        });
    }
    md5File(_path) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: md5File');
        });
    }
    resolve(..._paths) {
        throw new Error('Not implemented: resolve');
    }
    // Resolves the provided relative path to an absolute path within baseDir. The function
    // also checks that the absolute path is within baseDir, to avoid security issues.
    // It is expected that baseDir is a safe path (not user-provided).
    resolveRelativePathWithinDir(baseDir, relativePath) {
        const resolvedBaseDir = this.resolve(baseDir);
        const resolvedPath = this.resolve(baseDir, relativePath);
        if (resolvedPath.indexOf(resolvedBaseDir) !== 0)
            throw new Error(`Resolved path for relative path "${relativePath}" is not within base directory "${baseDir}" (Was resolved to ${resolvedPath})`);
        return resolvedPath;
    }
    getExternalDirectoryPath() {
        throw new Error('Not implemented: getExternalDirectoryPath');
    }
    isUsingAndroidSAF() {
        return false;
    }
    appendBinaryReadableToFile(path, readable) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = null;
            while ((data = readable.read()) !== null) {
                const buff = buffer_1.Buffer.from(data);
                const base64Data = buff.toString('base64');
                yield this.appendFile(path, base64Data, 'base64');
            }
        });
    }
    readDirStatsHandleRecursion_(basePath, stat, output, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options.recursive && stat.isDirectory()) {
                const subPath = `${basePath}/${stat.path}`;
                const subStats = yield this.readDirStats(subPath, options);
                for (let j = 0; j < subStats.length; j++) {
                    const subStat = subStats[j];
                    subStat.path = `${stat.path}/${subStat.path}`;
                    output.push(subStat);
                }
            }
            return output;
        });
    }
    findUniqueFilename(name, reservedNames = null, markdownSafe = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (reservedNames === null) {
                reservedNames = [];
            }
            let counter = 1;
            // On Windows, ./FiLe.md and ./file.md are equivalent file paths.
            // As such, to avoid overwriting reserved names, comparisons need to be
            // case-insensitive.
            reservedNames = reservedNames.map(name => name.toLowerCase());
            const isReserved = (testName) => {
                return reservedNames.includes(testName.toLowerCase());
            };
            const nameNoExt = (0, path_utils_1.filename)(name, true);
            let extension = (0, path_utils_1.fileExtension)(name);
            if (extension)
                extension = `.${extension}`;
            let nameToTry = nameNoExt + extension;
            while (true) {
                // Check if the filename does not exist in the filesystem and is not reserved
                const exists = (yield this.exists(nameToTry)) || isReserved(nameToTry);
                if (!exists)
                    return nameToTry;
                if (!markdownSafe) {
                    nameToTry = `${nameNoExt} (${counter})${extension}`;
                }
                else {
                    nameToTry = `${nameNoExt}-${counter}${extension}`;
                }
                counter++;
                if (counter >= 1000) {
                    nameToTry = `${nameNoExt} (${new Date().getTime()})${extension}`;
                    yield time_1.default.msleep(10);
                }
                if (counter >= 1100)
                    throw new Error('Cannot find unique filename');
            }
        });
    }
    removeAllThatStartWith(dirPath, filenameStart) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!filenameStart || !dirPath)
                throw new Error('dirPath and filenameStart cannot be empty');
            const stats = yield this.readDirStats(dirPath);
            for (const stat of stats) {
                if (stat.path.indexOf(filenameStart) === 0) {
                    yield this.remove(`${dirPath}/${stat.path}`);
                }
            }
        });
    }
    waitTillExists(path, timeout = 10000) {
        return __awaiter(this, void 0, void 0, function* () {
            const startTime = Date.now();
            while (true) {
                const e = yield this.exists(path);
                if (e)
                    return true;
                if (Date.now() - startTime > timeout)
                    return false;
                yield time_1.default.msleep(100);
            }
        });
    }
    // TODO: move out of here and make it part of joplin-renderer
    // or assign to option using .bind(fsDriver())
    cacheCssToFile(cssStrings) {
        return __awaiter(this, void 0, void 0, function* () {
            const cssString = Array.isArray(cssStrings) ? cssStrings.join('\n') : cssStrings;
            const cssFilePath = `${Setting_1.default.value('tempDir')}/${md5(escape(cssString))}.css`;
            if (!(yield this.exists(cssFilePath))) {
                yield this.writeFile(cssFilePath, cssString, 'utf8');
            }
            return {
                path: cssFilePath,
                mime: 'text/css',
            };
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    tarExtract(_options) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: tarExtract');
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    tarCreate(_options, _filePaths) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implemented: tarCreate');
        });
    }
}
exports.default = FsDriverBase;
//# sourceMappingURL=fs-driver-base.js.map