"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const SyncTargetRegistry_1 = require("../../SyncTargetRegistry");
const test_utils_1 = require("../../testing/test-utils");
const BaseItem_1 = require("../BaseItem");
const Folder_1 = require("../Folder");
const Setting_1 = require("../Setting");
const settingValidations_1 = require("./settingValidations");
describe('settingValidations', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    test('sync disabled items should prevent switching sync targets unless ignored', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder = yield Folder_1.default.save({ title: 'Test' });
        const noteCount = 5;
        const testNotes = yield (0, test_utils_1.createNTestNotes)(noteCount, folder);
        const syncTargetId = SyncTargetRegistry_1.default.nameToId('memory');
        Setting_1.default.setValue('sync.target', syncTargetId);
        for (const testNote of testNotes) {
            yield BaseItem_1.default.saveSyncDisabled(syncTargetId, testNote, 'Disabled reason');
        }
        const newSyncTargetId = SyncTargetRegistry_1.default.nameToId('dropbox');
        // Validation should fail with some error message.
        expect(yield (0, settingValidations_1.default)(['sync.target'], { 'sync.target': newSyncTargetId })).not.toBe('');
        // Should pass after dismissing all warnings
        for (const testNote of testNotes) {
            yield BaseItem_1.default.ignoreItemSyncWarning(syncTargetId, testNote);
        }
        expect(yield (0, settingValidations_1.default)(['sync.target'], { 'sync.target': newSyncTargetId })).toBe('');
    }));
});
//# sourceMappingURL=settingValidations.test.js.map