"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseModel_1 = require("../../BaseModel");
const trash_1 = require("../../services/trash");
const restoreItems_1 = require("../../services/trash/restoreItems");
const Folder_1 = require("../Folder");
const Note_1 = require("../Note");
const rootFolder = {
    id: '',
    deleted_time: 0,
    type_: BaseModel_1.ModelType.Folder,
};
exports.default = (noteIds, folderIds, targetFolderId) => __awaiter(void 0, void 0, void 0, function* () {
    let targetFolder;
    if (targetFolderId !== '') {
        targetFolder = yield Folder_1.default.load(targetFolderId, { fields: ['id', 'deleted_time'] });
    }
    else {
        targetFolder = rootFolder;
    }
    if (!targetFolder)
        throw new Error(`No such folder: ${targetFolderId}`);
    const defaultDeleteOptions = { toTrash: true, sourceDescription: 'onFolderDrop' };
    if (targetFolder.id !== (0, trash_1.getTrashFolderId)()) {
        defaultDeleteOptions.toTrashParentId = targetFolder.id;
    }
    function processList(itemType, itemIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const ModelClass = itemType === BaseModel_1.ModelType.Note ? Note_1.default : Folder_1.default;
            const items = yield ModelClass.byIds(itemIds, { fields: ['id', 'deleted_time', 'parent_id'] });
            for (const item of items) {
                if (item.id === targetFolder.id)
                    continue;
                if (targetFolder.deleted_time || targetFolder.id === (0, trash_1.getTrashFolderId)()) {
                    if (item.deleted_time && targetFolder.id === (0, trash_1.getTrashFolderId)()) {
                        yield ModelClass.delete(item.id, Object.assign(Object.assign({}, defaultDeleteOptions), { toTrashParentId: '' }));
                    }
                    else {
                        yield ModelClass.delete(item.id, defaultDeleteOptions);
                    }
                }
                else if (item.deleted_time && !targetFolder.deleted_time) {
                    yield (0, restoreItems_1.default)(itemType, [item], { targetFolderId: targetFolder.id });
                }
                else {
                    yield ModelClass.moveToFolder(item.id, targetFolderId);
                }
            }
        });
    }
    yield processList(BaseModel_1.ModelType.Note, noteIds);
    yield processList(BaseModel_1.ModelType.Folder, folderIds);
});
//# sourceMappingURL=onFolderDrop.js.map