"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const InteropService_Exporter_Md_1 = require("./InteropService_Exporter_Md");
const BaseModel_1 = require("../../BaseModel");
const NoteTag_1 = require("../../models/NoteTag");
const Tag_1 = require("../../models/Tag");
const frontMatter_1 = require("../../utils/frontMatter");
class InteropService_Exporter_Md_frontmatter extends InteropService_Exporter_Md_1.default {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
    prepareForProcessingItemType(itemType, itemsToExport) {
        const _super = Object.create(null, {
            prepareForProcessingItemType: { get: () => super.prepareForProcessingItemType }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.prepareForProcessingItemType.call(this, itemType, itemsToExport);
            if (itemType === BaseModel_1.default.TYPE_NOTE_TAG) {
                // Get tag list for each note
                const context = {
                    noteTags: {},
                };
                for (let i = 0; i < itemsToExport.length; i++) {
                    const it = itemsToExport[i].type;
                    if (it !== itemType)
                        continue;
                    const itemOrId = itemsToExport[i].itemOrId;
                    const noteTag = typeof itemOrId === 'object' ? itemOrId : yield NoteTag_1.default.load(itemOrId);
                    if (!noteTag)
                        continue;
                    if (!context.noteTags[noteTag.note_id])
                        context.noteTags[noteTag.note_id] = [];
                    context.noteTags[noteTag.note_id].push(noteTag.tag_id);
                }
                this.updateContext(context);
            }
            else if (itemType === BaseModel_1.default.TYPE_TAG) {
                // Map tag ID to title
                const context = {
                    tagTitles: {},
                };
                for (let i = 0; i < itemsToExport.length; i++) {
                    const it = itemsToExport[i].type;
                    if (it !== itemType)
                        continue;
                    const itemOrId = itemsToExport[i].itemOrId;
                    const tag = typeof itemOrId === 'object' ? itemOrId : yield Tag_1.default.load(itemOrId);
                    if (!tag)
                        continue;
                    context.tagTitles[tag.id] = tag.title;
                }
                this.updateContext(context);
            }
        });
    }
    getNoteExportContent_(modNote) {
        return __awaiter(this, void 0, void 0, function* () {
            let tagTitles = [];
            const context = this.context();
            if (context.noteTags[modNote.id]) {
                const tagIds = context.noteTags[modNote.id];
                // In some cases a NoteTag can still exist, while the Tag does not. In this case, tagTitles
                // for that tagId will return undefined, which can't be handled by the yaml library (issue #7782)
                tagTitles = tagIds.map((id) => context.tagTitles[id]).filter(e => !!e).sort();
            }
            return (0, frontMatter_1.serialize)(modNote, tagTitles);
        });
    }
}
exports.default = InteropService_Exporter_Md_frontmatter;
//# sourceMappingURL=InteropService_Exporter_Md_frontmatter.js.map