"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Note_1 = require("../../../models/Note");
const Api_1 = require("../Api");
const test_utils_1 = require("../../../testing/test-utils");
const Folder_1 = require("../../../models/Folder");
describe('routes/folders', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabase)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    test('should not include deleted folders in GET call', () => __awaiter(void 0, void 0, void 0, function* () {
        const api = new Api_1.default();
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({});
        yield api.route(Api_1.RequestMethod.DELETE, `folders/${folder1.id}`);
        const tree = yield api.route(Api_1.RequestMethod.GET, 'folders', { as_tree: 1 });
        expect(tree.length).toBe(1);
        expect(tree[0].id).toBe(folder2.id);
        const page = yield api.route(Api_1.RequestMethod.GET, 'folders');
        expect(page.items.length).toBe(1);
        expect(page.items[0].id).toBe(folder2.id);
    }));
    test('should be able to delete to trash', () => __awaiter(void 0, void 0, void 0, function* () {
        const api = new Api_1.default();
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({});
        const note1 = yield Note_1.default.save({ parent_id: folder1.id });
        const note2 = yield Note_1.default.save({ parent_id: folder2.id });
        const beforeTime = Date.now();
        yield api.route(Api_1.RequestMethod.DELETE, `folders/${folder1.id}`);
        yield api.route(Api_1.RequestMethod.DELETE, `folders/${folder2.id}`, { permanent: '1' });
        expect((yield Folder_1.default.load(folder1.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        expect(yield Folder_1.default.load(folder2.id)).toBeFalsy();
        expect((yield Note_1.default.load(note1.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        expect(yield Note_1.default.load(note2.id)).toBeFalsy();
    }));
});
//# sourceMappingURL=folders.test.js.map