"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupAutoDeletion = void 0;
const Logger_1 = require("@joplin/utils/Logger");
const Folder_1 = require("../../models/Folder");
const Setting_1 = require("../../models/Setting");
const Note_1 = require("../../models/Note");
const time_1 = require("@joplin/utils/time");
const shim_1 = require("../../shim");
const logger = Logger_1.default.create('permanentlyDeleteOldData');
const permanentlyDeleteOldItems = (ttl = null) => __awaiter(void 0, void 0, void 0, function* () {
    ttl = ttl === null ? Setting_1.default.value('trash.ttlDays') * time_1.Day : ttl;
    logger.info(`Processing items older than ${ttl}ms...`);
    if (!Setting_1.default.value('trash.autoDeletionEnabled')) {
        logger.info('Auto-deletion is not enabled - skipping');
        return;
    }
    const result = yield Folder_1.default.trashItemsOlderThan(ttl);
    logger.info('Items to permanently delete:', result);
    yield Note_1.default.batchDelete(result.noteIds, { sourceDescription: 'permanentlyDeleteOldItems' });
    // We only auto-delete folders if they are empty.
    for (const folderId of result.folderIds) {
        const noteIds = yield Folder_1.default.noteIds(folderId, { includeDeleted: true });
        if (!noteIds.length) {
            logger.info(`Deleting empty folder: ${folderId}`);
            yield Folder_1.default.delete(folderId);
        }
        else {
            logger.info(`Skipping non-empty folder: ${folderId}`);
        }
    }
});
const setupAutoDeletion = () => __awaiter(void 0, void 0, void 0, function* () {
    yield permanentlyDeleteOldItems();
    shim_1.default.setInterval(() => __awaiter(void 0, void 0, void 0, function* () {
        yield permanentlyDeleteOldItems();
    }), 18 * time_1.Hour);
});
exports.setupAutoDeletion = setupAutoDeletion;
exports.default = permanentlyDeleteOldItems;
//# sourceMappingURL=permanentlyDeleteOldItems.js.map