"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const time_1 = require("@joplin/utils/time");
const Folder_1 = require("../../models/Folder");
const Note_1 = require("../../models/Note");
const test_utils_1 = require("../../testing/test-utils");
const permanentlyDeleteOldItems_1 = require("./permanentlyDeleteOldItems");
const Setting_1 = require("../../models/Setting");
describe('permanentlyDeleteOldItems', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    it('should auto-delete old items', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({});
        const note1 = yield Note_1.default.save({ parent_id: folder1.id });
        const note2 = yield Note_1.default.save({});
        yield Folder_1.default.delete(folder1.id, { toTrash: true, deleteChildren: true });
        // First check that it doesn't auto-delete if it's not within the right interval
        yield (0, permanentlyDeleteOldItems_1.default)(time_1.Day);
        expect((yield Folder_1.default.load(folder1.id))).toBeTruthy();
        expect((yield Folder_1.default.load(folder2.id))).toBeTruthy();
        expect((yield Note_1.default.load(note1.id))).toBeTruthy();
        expect((yield Note_1.default.load(note2.id))).toBeTruthy();
        yield (0, time_1.msleep)(1);
        yield (0, permanentlyDeleteOldItems_1.default)(0);
        expect((yield Folder_1.default.load(folder1.id))).toBeFalsy();
        expect((yield Folder_1.default.load(folder2.id))).toBeTruthy();
        expect((yield Note_1.default.load(note1.id))).toBeFalsy();
        expect((yield Note_1.default.load(note2.id))).toBeTruthy();
    }));
    it('should not auto-delete non-empty folders', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({});
        const note1 = yield Note_1.default.save({ parent_id: folder1.id });
        const note2 = yield Note_1.default.save({});
        yield Folder_1.default.delete(folder1.id, { toTrash: true, deleteChildren: true });
        // Simulates a folder having been deleted a long time ago - so it should be deleted. But
        // since it contains a note it should not.
        yield Folder_1.default.save({ id: folder1.id, deleted_time: 1000 });
        yield (0, permanentlyDeleteOldItems_1.default)(time_1.Day);
        expect((yield Folder_1.default.load(folder1.id))).toBeTruthy();
        expect((yield Note_1.default.load(note1.id))).toBeTruthy();
        // Now both folders and items are within the deletion interval, so they should be both be
        // auto-deleted
        yield Note_1.default.save({ id: note1.id, deleted_time: 1000 });
        yield (0, permanentlyDeleteOldItems_1.default)(1);
        expect((yield Folder_1.default.load(folder1.id))).toBeFalsy();
        expect((yield Folder_1.default.load(folder2.id))).toBeTruthy();
        expect((yield Note_1.default.load(note1.id))).toBeFalsy();
        expect((yield Note_1.default.load(note2.id))).toBeTruthy();
    }));
    it('should not do anything if auto-deletion is not enabled', () => __awaiter(void 0, void 0, void 0, function* () {
        Setting_1.default.setValue('trash.autoDeletionEnabled', false);
        const folder1 = yield Folder_1.default.save({});
        yield Folder_1.default.delete(folder1.id, { toTrash: true });
        yield (0, time_1.msleep)(1);
        yield (0, permanentlyDeleteOldItems_1.default)(0);
        expect(yield Folder_1.default.count()).toBe(1);
    }));
});
//# sourceMappingURL=permanentlyDeleteOldItems.test.js.map