"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const BaseModel_1 = require("../../BaseModel");
const Folder_1 = require("../../models/Folder");
const Note_1 = require("../../models/Note");
const object_1 = require("@joplin/utils/object");
const restoreItems = (itemType, itemsOrIds, options = null) => __awaiter(void 0, void 0, void 0, function* () {
    if (!itemsOrIds.length)
        return;
    options = Object.assign({ targetFolderId: null, useRestoreFolder: false }, options);
    const ModelClass = itemType === BaseModel_1.ModelType.Note ? Note_1.default : Folder_1.default;
    let items = [];
    if (typeof itemsOrIds[0] === 'string') {
        items = yield ModelClass.byIds(itemsOrIds, { fields: ['id', 'parent_id', 'deleted_time'] });
    }
    else {
        items = itemsOrIds;
    }
    let restoreFolder = null;
    for (const item of items) {
        (0, object_1.checkObjectHasProperties)(item, ['id', 'parent_id']);
        let itemParentId = item.parent_id;
        const parentItem = yield Folder_1.default.load(item.parent_id, { fields: ['id', 'deleted_time'] });
        if (!parentItem || parentItem.deleted_time) {
            if (options.useRestoreFolder) {
                if (!restoreFolder)
                    restoreFolder = yield (0, _1.getRestoreFolder)();
                itemParentId = restoreFolder.id;
            }
            else {
                itemParentId = '';
            }
        }
        if (options.targetFolderId !== null)
            itemParentId = options.targetFolderId;
        let toSave = null;
        if (itemType === BaseModel_1.ModelType.Note) {
            // We need to preview conflicts -- they can be trashed.
            toSave = yield Note_1.default.preview(item.id, { excludeConflicts: false });
        }
        else {
            toSave = yield Folder_1.default.load(item.id);
        }
        toSave = Object.assign(Object.assign({}, toSave), { deleted_time: 0, updated_time: Date.now(), parent_id: itemParentId });
        yield ModelClass.save(toSave, {
            autoTimestamp: false,
        });
        if (itemType === BaseModel_1.ModelType.Folder) {
            const childrenFolderIds = yield Folder_1.default.childrenIds(item.id, { includeDeleted: true });
            const childrenFolders = yield Folder_1.default.byIds(childrenFolderIds, { fields: ['id', 'parent_id', 'deleted_time'] });
            const deletedChildrenFolders = childrenFolders.filter(f => !!f.deleted_time);
            yield restoreItems(BaseModel_1.ModelType.Folder, deletedChildrenFolders);
            const notes = yield Folder_1.default.notes(item.id, {
                fields: ['id', 'parent_id'],
                includeDeleted: true,
            });
            yield restoreItems(BaseModel_1.ModelType.Note, notes);
        }
    }
});
exports.default = restoreItems;
//# sourceMappingURL=restoreItems.js.map