"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const BaseModel_1 = require("../../BaseModel");
const Folder_1 = require("../../models/Folder");
const Note_1 = require("../../models/Note");
const test_utils_1 = require("../../testing/test-utils");
const restoreItems_1 = require("./restoreItems");
describe('restoreItems', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
    }));
    it('should restore notes', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder = yield Folder_1.default.save({});
        const note1 = yield Note_1.default.save({ parent_id: folder.id });
        const note2 = yield Note_1.default.save({ parent_id: folder.id });
        yield Note_1.default.delete(note1.id, { toTrash: true });
        yield Note_1.default.delete(note2.id, { toTrash: true });
        expect((yield Folder_1.default.noteIds(folder.id)).length).toBe(0);
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Note, [yield Note_1.default.load(note1.id), yield Note_1.default.load(note2.id)]);
        expect((yield Folder_1.default.noteIds(folder.id)).length).toBe(2);
    }));
    it('should restore folders and included notes', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder1 = yield Folder_1.default.save({});
        const note1 = yield Note_1.default.save({ parent_id: folder1.id });
        const note2 = yield Note_1.default.save({ parent_id: folder1.id });
        yield Folder_1.default.delete(folder1.id, { toTrash: true });
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Folder, [yield Folder_1.default.load(folder1.id)]);
        expect((yield Folder_1.default.load(folder1.id)).deleted_time).toBe(0);
        expect((yield Note_1.default.load(note1.id)).deleted_time).toBe(0);
        expect((yield Note_1.default.load(note2.id)).deleted_time).toBe(0);
    }));
    it('should restore folders and sub-folders', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({ parent_id: folder1.id });
        const note1 = yield Note_1.default.save({ parent_id: folder2.id });
        const note2 = yield Note_1.default.save({ parent_id: folder2.id });
        const beforeTime = Date.now();
        yield Folder_1.default.delete(folder1.id, { toTrash: true, deleteChildren: true });
        expect((yield Folder_1.default.load(folder1.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        expect((yield Folder_1.default.load(folder2.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        expect((yield Note_1.default.load(note1.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        expect((yield Note_1.default.load(note2.id)).deleted_time).toBeGreaterThanOrEqual(beforeTime);
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Folder, [yield Folder_1.default.load(folder1.id)]);
        expect((yield Folder_1.default.load(folder1.id)).deleted_time).toBe(0);
        expect((yield Folder_1.default.load(folder2.id)).deleted_time).toBe(0);
        expect((yield Note_1.default.load(note1.id)).deleted_time).toBe(0);
        expect((yield Note_1.default.load(note2.id)).deleted_time).toBe(0);
    }));
    it('should restore a note, even if the parent folder no longer exists', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder = yield Folder_1.default.save({});
        const note = yield Note_1.default.save({ parent_id: folder.id });
        yield Folder_1.default.delete(folder.id, { toTrash: true });
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Note, [yield Note_1.default.load(note.id)]);
        const noteReloaded = yield Note_1.default.load(note.id);
        expect(noteReloaded.parent_id).toBe('');
    }));
    it('should restore a folder, even if the parent folder no longer exists', () => __awaiter(void 0, void 0, void 0, function* () {
        const folder1 = yield Folder_1.default.save({});
        const folder2 = yield Folder_1.default.save({});
        const note = yield Note_1.default.save({ parent_id: folder2.id });
        yield Folder_1.default.delete(folder1.id, { toTrash: true });
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Note, [yield Folder_1.default.load(folder2.id)]);
        const folderReloaded2 = yield Folder_1.default.load(folder2.id);
        const noteReloaded = yield Note_1.default.load(note.id);
        expect(folderReloaded2.parent_id).toBe('');
        expect(noteReloaded.parent_id).toBe(folderReloaded2.id);
    }));
    it('should restore a conflict', () => __awaiter(void 0, void 0, void 0, function* () {
        const note = yield Note_1.default.save({ is_conflict: 1, title: 'Test' });
        yield Note_1.default.delete(note.id, { toTrash: true });
        yield (0, restoreItems_1.default)(BaseModel_1.ModelType.Note, [yield Note_1.default.load(note.id)]);
        const noteReloaded = yield Note_1.default.load(note.id);
        expect(noteReloaded.title).toBe('Test');
        expect(noteReloaded.is_conflict).toBe(1);
        expect(noteReloaded.deleted_time).toBe(0);
    }));
});
//# sourceMappingURL=restoreItems.test.js.map