"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("@joplin/utils/Logger");
const test_utils_1 = require("../testing/test-utils");
const Note_1 = require("../models/Note");
const ActionLogger_1 = require("./ActionLogger");
const Setting_1 = require("../models/Setting");
const fs_extra_1 = require("fs-extra");
const getLogPath = () => `${Setting_1.default.value('profileDir')}/log.txt`;
const logContainsEntryWith = (...terms) => __awaiter(void 0, void 0, void 0, function* () {
    const lines = (yield (0, fs_extra_1.readFile)(getLogPath(), 'utf8')).split('\n');
    for (const line of lines) {
        if (terms.every(t => line.includes(t))) {
            return true;
        }
    }
    return false;
});
describe('ActionLogger', () => {
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, test_utils_1.setupDatabaseAndSynchronizer)(1);
        yield (0, test_utils_1.switchClient)(1);
        const logPath = getLogPath();
        if (yield (0, fs_extra_1.pathExists)(logPath)) {
            yield (0, fs_extra_1.remove)(logPath);
        }
        yield (0, fs_extra_1.writeFile)(logPath, '', 'utf8');
        const logger = new Logger_1.default();
        logger.addTarget(Logger_1.TargetType.File, { path: logPath });
        logger.setLevel(Logger_1.LogLevel.Info);
        Logger_1.default.initializeGlobalLogger(logger);
    }));
    it('should log deletions', () => __awaiter(void 0, void 0, void 0, function* () {
        const note = yield Note_1.default.save({ title: 'MyTestNote' });
        yield Note_1.default.delete(note.id, { toTrash: false });
        yield Logger_1.default.globalLogger.waitForFileWritesToComplete_();
        expect(yield logContainsEntryWith('DeleteAction', note.id, note.title)).toBe(true);
    }));
    it('should be possible to disable ActionLogger globally', () => __awaiter(void 0, void 0, void 0, function* () {
        const note1 = yield Note_1.default.save({ title: 'testNote1' });
        const note2 = yield Note_1.default.save({ title: 'testNote2' });
        ActionLogger_1.default.enabled = true;
        yield Note_1.default.delete(note1.id, { toTrash: false });
        ActionLogger_1.default.enabled = false;
        yield Note_1.default.delete(note2.id, { toTrash: false });
        ActionLogger_1.default.enabled = true;
        yield Logger_1.default.globalLogger.waitForFileWritesToComplete_();
        expect(yield logContainsEntryWith('DeleteAction', note1.id)).toBe(true);
        expect(yield logContainsEntryWith('DeleteAction', note2.id)).toBe(false);
    }));
});
//# sourceMappingURL=ActionLogger.test.js.map